package com.example.nto.controller;

import com.example.nto.dto.EmployeeDto;
import com.example.nto.dto.EnterDto;
import com.example.nto.exception.NoRequestBodyException;
import com.example.nto.service.EmployeeService;
import com.example.nto.service.EnterService;
import lombok.RequiredArgsConstructor;
import com.example.nto.dto.CodeDto;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.security.core.Authentication;
import org.springframework.web.bind.annotation.*;

import java.util.List;

@RestController
@RequiredArgsConstructor
@RequestMapping("/api")
public class EmployeeController {
    private final EmployeeService employeeService;
    private final EnterService enterService;

    @GetMapping("/login/{login}")
    public ResponseEntity<String> getByUsername(@PathVariable String login) {
        if (login == null) {
            throw new NoRequestBodyException("No or wrong request body!");
        }

        EmployeeDto employeeDto = employeeService.getByLogin(login);
        return ResponseEntity.ok(employeeDto.getLogin());
    }

    @GetMapping("/login")
    public ResponseEntity<EmployeeDto> login(Authentication authentication) {
        if (authentication == null) {
            throw new NoRequestBodyException("No or wrong request body!");
        }
        return ResponseEntity.ok(employeeService.getByLogin(authentication.getName()));
    }

    @GetMapping("/{login}/info")
    public ResponseEntity<EmployeeDto> infoEmployee(@PathVariable String login) {
        EmployeeDto employeeDto = employeeService.getByLogin(login);
        return ResponseEntity.ok(employeeDto);
    }

    @PatchMapping("/{login}/open")
    public ResponseEntity<Boolean> openEmployee(@PathVariable String login, @RequestBody CodeDto value) {
        String code = value.getValue();
        return ResponseEntity.ok(employeeService.enterEmployee(login, code));
    }

    @GetMapping("/{login}/entrances")
    public ResponseEntity<List<EnterDto>> infoEnters(@PathVariable String login) {
        List<EnterDto> enters = enterService.getByLogin(login);
        return ResponseEntity.ok(enters);
    }

    @PatchMapping("/{login}/block")
    public ResponseEntity<String> blockEmployee(@PathVariable String login) {
        return ResponseEntity.ok(employeeService.blockEmployee(login));
    }

    @PatchMapping("/{login}/unblock")
    public ResponseEntity<String> unblockEmployee(@PathVariable String login) {
        return ResponseEntity.ok(employeeService.unblockEmployee(login));
    }
}
