package com.example.nto.service.impl;

import com.example.nto.dto.EmployeeDto;
import com.example.nto.entity.Code;
import com.example.nto.entity.Employee;
import com.example.nto.entity.Enter;
import com.example.nto.exception.*;
import com.example.nto.repository.CodeRepository;
import com.example.nto.repository.EnterRepository;
import com.example.nto.service.EmployeeService;
import com.example.nto.repository.EmployeeRepository;

import com.example.nto.utils.EmployeeMapper;
import liquibase.pro.packaged.B;
import lombok.RequiredArgsConstructor;
import org.springframework.stereotype.Service;

import java.sql.Timestamp;
import java.time.LocalDateTime;
import java.util.Optional;

@Service
@RequiredArgsConstructor
public class EmployeeServiceImpl implements EmployeeService {

    private final EmployeeRepository employeeRepository;
    private final EnterRepository enterRepository;
    private final CodeRepository codeRepository;


    @Override
    public EmployeeDto getByLogin(String username) {
        Optional<Employee> employee = employeeRepository.findByLogin(username);
        if (employee.isEmpty()) {
            throw new UserNotFoundException("User with name " + username + " not found!");
        }
        return EmployeeMapper.convertToDto(employee.get());
    }


    @Override
    public String blockEmployee(String login) {
        Optional<Employee> employeeOp = employeeRepository.findByLogin(login);
        if (employeeOp.isPresent()) {
            Employee employee = employeeOp.get();
            employee.setAccess(false);
            return "blocked";
        }
        throw new UserNotFoundException("user with login " + login + " not found");
    }

    @Override
    public String unblockEmployee(String login) {
        Optional<Employee> employeeOp = employeeRepository.findByLogin(login);
        if (employeeOp.isPresent()) {
            Employee employee = employeeOp.get();
            employee.setAccess(true);
            return "unblocked";
        }
        throw new UserNotFoundException("user with login " + login + " not found");
    }

    @Override
    public Boolean enterEmployee(String login, String value) {
        Employee employee = EmployeeMapper.convertFromDto(getByLogin(login));
        if (employee != null) {
            Optional<Code> codeOp = codeRepository.findByValue(value);
            if (codeOp.isPresent()) {
                Code code = codeOp.get();
                if (employee.isAccess()) {
                    Enter enter = new Enter();
                    enter.setEmployee(employee);
                    enter.setCode(code);
                    enter.setEntered_at(Timestamp.valueOf(LocalDateTime.now()));
                    enter.setEnterType("Phone");
                    enterRepository.save(enter);
                    return true;
                } else {
                    return false;
                }
            } else {
                throw new CodeNotFoundException("Неверный код");
            }
        } else {
            throw new UserNotFoundException("логина не существует или неверный");
        }
    }
}